package ap.data;

/**
 * Title:        Klasse CStockAgentMsgStrategy<p>
 * Description:  Nachricht an einen StockAgent, der ihn zum Beenden
 *		 veranlasst<p>
 * Copyright:    Copyright (c) 2001<p>
 * Company:      Ametas Praktikum, Uni Frankfurt, Fachbereich Biologie und Informatik, WS2000/2001<p>
 * @author Martin Klossek, Fabian Wleklinski
 * @version 1.0
 */

public class CStockAgentMsgStrategy extends CStockAgentMsg implements AMETAS.data.AMETASSendable {

        /**
         * Der numerische Bezeichner fr die Strategie, der Vorgabewert
         * (die Basisstrategie) ist 0.
         */
        private int m_iStrategy;

        /**
         * Die Werte fr die Parameter (Die Bedeutung ist von der
         * gewhlten Strategie abhngig)
         */
        private double[] m_dParams;

        /**
         * Die Anzahl der Parameterwerte, die gehalten werden knnen. Entspricht
         * dem Wert, der beim Aufruf des Konstruktors angegeben wird, sofern er
         * angegeben wird, bzw. dem Vorgabewert sonst.
         */
        private int m_iParamsCount;

        /**
         * Der Vorgabewert fr die Anzahl der Parameterwerte, die gehalten werden
         * knnen. Wird nur dann verwendet, wenn beim Aufruf des Konstruktors kein anderer
         * Wert spezifiziert wird.
         */
        private final static int m_iParamsCountDefault = 5;

        /**
         * Konstruktor der Klasse
         */
        public CStockAgentMsgStrategy() {
                this( m_iParamsCountDefault );
        }

        /**
         * Konstruktor der Klasse
         * @param iParamsCount Die Anzahl der Parameter die gehalten werden sollen
         */
        public CStockAgentMsgStrategy( int iParamsCount ) {
                this.redimParamField( iParamsCount );
        }

        /**
         * Konstruktor der Klasse
         * @param iStrategy Der numerische Bezeicher der Strategie
         * @param dParams Ein Feld mit den Werten der Parameter
         */
        public CStockAgentMsgStrategy( int iStrategy, double[] dParams ) {
                m_dParams = dParams;
                m_iParamsCount = dParams.length;
                m_iStrategy = iStrategy;
        }

        /**
         * Redimensioniert das Feld fr die Aufnahme der Parameterwerte
         * @param iParamsCount Die Anzahl der Parameter die gehalten werden sollen
         */
        private void redimParamField( int iParamsCount ) {
                /* Fabian Wleklinski: Das Feld erzeugen, in dem die Parameterwerte
                gehalten werden */
                this.m_dParams = new double[ iParamsCount ];
        }

        /**
         * Liefert die Anzahl der Parameter zurck, die gehalten werden (sollen)
         */
        public int getParamsCount() {
                return this.m_iParamsCount;
        }

        /**
         * Liefert den numerischen Bezeichner der Strategie zurck
         */
        public int getStrategy() {
                return m_iStrategy;
        }

        /**
         * Setzt die Strategie anhand des bergebenen, numerischen Bezeichners
         * @param iStrategy Der numerische Bezeichern der zu setzenden Strategie
         */
        public void setStrategy( int iStrategy ) {
                m_iStrategy = iStrategy;
        }

        /**
         * Liefert den Wert des Parameter mit dem bergebenen Index
         * @param iParamIndex Der Index des Parameters, dessen Wert zurckgegeben werden soll
         * @return Der Wert des Parameters mit dem bergebenen Index
         */
        public double getParam( int iParamIndex ) {
                return this.m_dParams[ iParamIndex ];
        }

        /**
         * Weist dem Parameter mit dem bergebenen Index den bergebenen Wert zu
         * @param iParamIndex Der Index des Parameters, dem der Wert zugewiesen werden soll
         * @param dParamValue Der dem Parameter zuzuweisende Wert
         */
        public void setParam( int iParamIndex, double dParamValue ) {
                this.m_dParams[ iParamIndex ] = dParamValue;
        }

        /**
         * Die Funktion &quot;getData()&quot; wird vom Interface &quot;AMETASSendable&quot;
         * gefordert. Sie dient dazu, im Falle einer Serialisierung der Klasse (bei einer
         * Migration des Agenten) alle enthaltenen Daten/Variablen/Members in generische
         * Objekte zu verwandeln.
         */
        public Object[] getData() {

                /* Fabian Wleklinski: Die Anzahl der Parameter laden: */
                int iParamsCount = this.getParamsCount();

                /* Fabian Wleklinski: Es muss ein Feld von generischen Objekten zurckgegeben
                werden, dies fordert das "AMETASSendable"-Interface. Dieses Feld wird nun
                erzeugt: */
                Object[] aobjData = new Object[ iParamsCount + 1 ];

                /* Fabian Wleklinski: Den numerischen Bezeichner der Strategie in dieses
                Feld schreiben: */
                aobjData[ 0 ] = new Integer( this.getStrategy() );

                /* Fabian Wleklinski: Nun alle Daten/Variablen/Members in dieses Feld
                schreiben: */
                for (int i=0; i < iParamsCount; i++) {
                        aobjData[ i+1 ] = new Double( this.getParam( i ) );
                }

                /* Fabian Wleklinski: Nun dieses Feld zurckgeben: */
                return aobjData;
        }

        /**
         * Die Funktion &quot;setData()&quot; wird vom Interface &quot;AMETASSendable&quot;
         * gefordert. Sie dient dazu, im Falle einer Serialisierung der Klasse (bei einer
         * Migration des Agenten) alle enthaltenen Daten/Variablen/Members aus generischen
         * Objekten zu lesen.
         */
        public void setData( Object[] aobjData ) {

                /* Fabian Wleklinski: Nun alle Daten/Variablen/Members aus dem
                bergebenen Feld lesen, und in die entsprechenden Variablen eintragen: */

                /* Fabian Wleklinski: Den numerischen Bezeichner der Strategie lesen: */
                this.setStrategy( ((Integer) aobjData[ 0 ]).intValue() );

                /* Fabian Wleklinski: Die Anzahl der Parameter laden: */
                int iParamsCount = aobjData.length - 1;

                /* Fabian Wleklinski: Das Feld fr die Aufnahme der Parameterwerte lschen
                und gem der neuen Parameteranzahl redimensionieren: */
                this.redimParamField( iParamsCount );

                for (int i=0; i < iParamsCount; i++) {
                        this.setParam( i, ((Double) aobjData[ i+1 ]).doubleValue() );
                }

                /* Fabian Wleklinski: Den neuen Wert fr die Anzahl der Parameter abspeichern: */
                this.m_iParamsCount = iParamsCount;

        }

}
